unit ugter_combinado_horario;

interface


{xDEFINE dbgc}
uses
  SysUtils, Classes, xmatdefs, uGTer, unodos,
  Math,
  uglobs,
  umipsimplex,
  usimplex,
  ufichasLPD,
  ufechas,
  ucosa, uCosaConNombre,
  ugter_combinado,
  uEstados,
  uconstantesSimSEE, uFuentesAleatorias;

resourcestring
  rsGeneradorTermicoCombinadoHorario = 'Generador térmico combinado horario';

type

  { TFichaGTer_combinado_horario }

  TFichaGTer_combinado_horario = class(TFichaGTer_combinado)

  public

    (**************************************************************************)
    (*               A T R I B U T O S   P E R S I S T E N T E S              *)
    (**************************************************************************)

    horasPurga: NReal;
    horasTVPleno: NReal;
    NDisc: integer;
    (**************************************************************************)

    constructor Create(capa: integer; fecha: TFecha; periodicidad: TPeriodicidad;
      PMin_TG, PMax_TG: NReal; cv_min_TG, cv_TG, cv_NoCombustible_TG: NReal;
      disp_TG: NReal; tRepHoras_TG: NReal; PMin_TV, PMax_TV: NReal;
      cv_min_TV, cv_TV, cv_NoCombustible_TV: NReal; disp_TV: NReal;
      tRepHoras_TV: NReal; factorPotenciaTVdivTG: NReal;
      indicePreciosPorCombustible: TFuenteAleatoria; bornePreciosPorCombustible: string;
      HayRestriccionEmaxPasoDeTiempo: boolean; EmaxPasoDeTiempo: NReal;
      PagoPorDisponibilidad_USD_MWh, PagoPorEnergia_USD_MWh: NReal;
      horasPurga: NReal; horasTVPleno: NReal; NDisc: integer);

    function Rec: TCosa_RecLnk; override;
    procedure BeforeRead(version, id_hilo: integer); override;
    procedure AfterRead(f: TArchiTexto); override;

    procedure generarLineaResumen(var archi: TextFile); override;
    function infoAd_: string; override;
    procedure Free; override;


  published

  end;


  { TGTer_combinado_horario }

  TGTer_combinado_horario = class(TGTer)
  public

    pa: TFichaGTer_combinado_horario;
    EstadoTimer: NReal;
    NDisc: integer;
    resCode: integer;
    xrpos: NReal;
    dCFdX, dCFdX_inc, dCFdx_dec: NReal;
    // afectación del costo futuro por movimiento del estado.

    (******** Esta es la variable de estado ******)
    X_timer: NReal;
    // timer que empieza a correr cuando se cierra el ciclo al inicio del paso
    Xs_timer: NReal;
    // timer que empieza a correr cuando se cierra el ciclo al final del paso

    horasPurga: NReal; //horas que demora en purgar antes de poder prender la TV
    horasTVPleno: NReal; //horas que demora en llegar a pleno la TV

    umbralDespachoTV: NReal;
    factorTV: NReal;   //factor TV/TG si se puede despachar TV o 0 sino

    //variable para saber en que parte del ciclo estoy
    //0 cuando se esta purgando, si purga_TV es 0 y X_timer es 0, el ciclo esta abierto
    //1 cuando se esta subiendo las TG con ciclo cerrado
    //2 cuando puedo estar a pleno
    purga_TV: integer;
    //_SC sin calentar (sin calentar calderas) _CC calentando caldera
    //divido la variable de la potencia de las TG en 2, para diferenciar
    //si se calienta o no la caldera
    BMaxTG_SC, BMaxTG_CC, BMaxTV: NReal; // MW, cota sup de P-Pmin
    PMaxTG: NReal;
    PMinTG: NReal; //MW, mínimo
    PMinTV: NReal; // MW, minimo de las TV

    NMaquinasDespachadasTG, NMaquinasDespachadasTV: integer;

    //pendiente con la que sube y baja la temperatura de la caldera
    //    pendTempSubida, pendTempBajada: NReal;
    pendTV: NReal;

    c0TG, c0TV: NReal;
    //Costo en USD/h por máquina por estar operando en el mínimo técnico
    cvTG, cvTV: NReal; //USD/MWh usado para la optimización del paso.

    // resultado de los sorteos de disponibilidad y del mantenimiento programado
    NMaquinasDisponiblesTG, NMaquinasDisponiblesTV: integer;
    PMaxDisponible_TG, PMaxDisponible_TV: NReal;
    PMaxDisponible_Central: NReal;

    // indices a los tramos de variables
    ivar_ATG_SC, ivar_BTG_SC, ivar_ATG_CC, ivar_BTG_CC,
    ivar_ATV, ivar_BTV, ivar_AAcople, ivar_Xs_timer: integer;

    // indices a las restricciones
    jres_dinTimer, jres_Acople_BTG_SC, jres_Acople_BTG_CC,
    jres_Acople_BTV, jres_Acople_TVxTG, jres_Acople_TimerxBTG_SC,
    jres_Acople_TimerxBTG_CC, jres_extTimer, jres_EMaxPasoDeTiempo,
    jres_Ultima, jaux: integer;

    //Cuanta potencia despacho en cada hora el generador en promedio
    potMedia_despachada: NReal;

    constructor Create(capa: integer; nombre: string; nacimiento, muerte: TFecha;
      lpdUnidades, lpd: TFichasLPD; nodo: TNodo;
  flg_CalcularGradienteDeInversion: boolean; TonCO2xMWh: NReal; LowCostMustRun,
  CleanDevelopmentMechanism: boolean; xFuenteIdxP: TFuenteAleatoria;
  xBorneIdxP: string; EstadoTimer: NReal);


    function Rec: TCosa_RecLnk; override;
    procedure BeforeRead(version, id_hilo: integer); override;
    procedure AfterRead(f: TArchiTexto); override;

    procedure PrepararMemoria(Catalogo: TCatalogoReferencias; globs: TGlobs); override;
    procedure RegistrarParametrosDinamicos(CatalogoReferencias: TCatalogoReferencias);
      override;

    function PotenciaFirme: NReal; override;

    function InfoAd_: string; override;
    class function DescClase: string; override;

    function get_pa_FD(kTipoUnidad: integer): NReal; override;
    function get_pa_TMR(kTipoUnidad: integer): NReal; override;

    procedure SorteosDelPaso(sortear: boolean); override;
    procedure PrepararPaso_ps; override;
    procedure Sim_Paso_Fin; override;

    procedure opt_nvers(var ivar, ivae, ires: integer); override;
    procedure opt_nvers_TV_indisponible(var ivar, ivae, ires: integer);
    procedure opt_nvers_abierto_purga(var ivar, ivae, ires: integer);
    procedure opt_nvers_combinado(var ivar, ivae, ires: integer);

    procedure opt_cargue(s: TSimplex); override;
    procedure opt_cargue_TV_indisponible(s: TSimplex);
    procedure opt_cargue_abierto_purga(s: TSimplex);
    procedure opt_cargue_combinado(s: TSimplex);

    procedure opt_fijarRestriccionesDeCaja(s: TSimplex); override;
    procedure opt_fijarRestriccionesDeCaja_TV_indisponible(s: TSimplex);
    procedure opt_fijarRestriccionesDeCaja_abierto_purga(s: TSimplex);
    procedure opt_fijarRestriccionesDeCaja_transicion(s: TSimplex);
    procedure opt_fijarRestriccionesDeCaja_combinado(s: TSimplex);

    procedure opt_leerSolucion(s: TSimplex); override;
    procedure opt_leerSolucion_TV_indisponible(s: TSimplex);
    procedure opt_leerSolucion_abierto_purga(s: TSimplex);
    procedure opt_leerSolucion_combinado(s: TSimplex);

    procedure Sim_Cronica_Inicio; override;

    function getNombreVar(ivar: integer; var nombre: string): boolean; override;
    function getNombreVar_TV_indisponible(ivar: integer; var nombre: string): boolean;
    function getNombreVar_abierto_purga(ivar: integer; var nombre: string): boolean;
    function getNombreVar_combinado(ivar: integer; var nombre: string): boolean;

    function getNombreRes(ires: integer; var nombre: string): boolean; override;
    function getNombreRes_TV_indisponible(ires: integer; var nombre: string): boolean;
    function getNombreRes_abierto_purga(ires: integer; var nombre: string): boolean;
    function getNombreRes_combinado(ires: integer; var nombre: string): boolean;

    procedure Free; override;

    procedure PubliVars; override;
    procedure dump_Variables(var f: TextFile; charIndentacion: char); override;

    class function TipoFichaLPD: TClaseDeFichaLPD; override;

    procedure EvolucionarEstado; override;

    procedure PosicionarseEnEstrellita; override;
    procedure ActualizarEstadoGlobal(flg_Xs: boolean); override;

    procedure optx_nvxs(var ixr, ixd, iauxr, iauxd: integer); override;
    procedure optx_RegistrarVariablesDeEstado(adminEstados: TAdminEstados); override;

  end;

procedure AlInicio;
procedure AlFinal;

implementation

//------------------------------
// Métodos de TFichaGter_combinado_horario
//==============================
constructor TFichaGTer_combinado_horario.Create(capa: integer;
  fecha: TFecha; periodicidad: TPeriodicidad; PMin_TG, PMax_TG: NReal;
  cv_min_TG, cv_TG, cv_NoCombustible_TG: NReal; disp_TG: NReal;
  tRepHoras_TG: NReal; PMin_TV, PMax_TV: NReal;
  cv_min_TV, cv_TV, cv_NoCombustible_TV: NReal; disp_TV: NReal;
  tRepHoras_TV: NReal; factorPotenciaTVdivTG: NReal;
  indicePreciosPorCombustible: TFuenteAleatoria; bornePreciosPorCombustible: string;
  HayRestriccionEmaxPasoDeTiempo: boolean; EmaxPasoDeTiempo: NReal;
  PagoPorDisponibilidad_USD_MWh, PagoPorEnergia_USD_MWh: NReal;
  horasPurga: NReal; horasTVPleno: NReal; NDisc: integer);

begin
  inherited Create(capa, fecha, periodicidad);

  self.PMin_TG := PMin_TG;
  self.PMax_TG := PMax_TG;
  self.cv_min_TG := cv_min_TG;
  self.cv_TG := cv_TG;
  self.cv_NoCombustible_TG := cv_NoCombustible_TG;
  self.disp_TG := disp_TG;
  self.tRepHoras_TG := tRepHoras_TG;

  self.disp_TV := disp_TV;
  self.tRepHoras_TV := tRepHoras_TV;

  self.PMin_TV := PMin_TV;
  self.PMax_TV := PMax_TV;
  self.cv_min_TV := cv_min_TV;
  self.cv_TV := cv_TV;
  self.cv_NoCombustible_TV := cv_NoCombustible_TV;
  self.flg_TV_OnOffPorPaso := flg_TV_OnOffPorPaso;

  self.factorPotenciaTVdivTG := factorPotenciaTVdivTG;

  self.indicePreciosPorCombustible := indicePreciosPorCombustible;
  self.bornePreciosPorCombustible := bornePreciosPorCombustible;
  self.HayRestriccionEmaxPasoDeTiempo := HayRestriccionEmaxPasoDeTiempo;
  self.EmaxPasoDeTiempo := EmaxPasoDeTiempo;
  self.PagoPorDisponibilidad_USD_MWh := PagoPorDisponibilidad_USD_MWh;
  self.PagoPorEnergia_USD_MWh := PagoPorEnergia_USD_MWh;


  self.horasPurga := horasPurga;
  self.horasTVPleno := horasTVPleno;
  self.NDisc := NDisc;

end;

function TFichaGTer_combinado_horario.Rec: TCosa_RecLnk;
begin
  Result := inherited Rec;
  Result.addCampoDef('PMin_TG', PMin_TG);
  Result.addCampoDef('PMax_TG', PMax_TG);
  Result.addCampoDef('cv_min_TG', cv_min_TG);
  Result.addCampoDef('cv_TG', cv_TG);
  Result.addCampoDef('cv_NoCombustible_TG', cv_NoCombustible_TG, 111);
  Result.addCampoDef('disp_TG', disp_TG);
  Result.addCampoDef('tRepHoras_TG', tRepHoras_TG);
  Result.addCampoDef('disp_TV', disp_TV, 0, 98);
  Result.addCampoDef('tRepHoras_TV', tRepHoras_TV, 0, 98);
  Result.addCampoDef('PMin_CC', PMin_TV, 0, 98);
  Result.addCampoDef('PMax_CC', PMax_TV, 0, 98);
  Result.addCampoDef('cv_min_CC', cv_min_TV, 0, 98);
  Result.addCampoDef('cv_CC', cv_TV, 0, 98);
  Result.addCampoDef('PMin_TV', PMin_TV, 98);
  Result.addCampoDef('PMax_TV', PMax_TV, 98);
  Result.addCampoDef('cv_min_TV', cv_min_TV, 98);
  Result.addCampoDef('cv_TV', cv_TV, 98);
  Result.addCampoDef('cv_NoCombustible_TV', cv_NoCombustible_TV, 111);
  Result.addCampoDef('disp_TV', disp_TV, 98);
  Result.addCampoDef('tRepHoras_TV', tRepHoras_TV, 98);
  Result.addCampoDef('flg_TV_OnOffPorPaso', flg_TV_OnOffPorPaso, 113);
  Result.addCampoDef('factorPotenciaTVdivTG', factorPotenciaTVdivTG);
  Result.addCampoDef_ref('indicePreciosPorCombustible',
    TCosa(indicePreciosPorCombustible), self);
  Result.addCampoDef('bornePreciosPorCombustible', bornePreciosPorCombustible);
  Result.addCampoDef('HayRestriccionEmaxPasoDeTiempo', HayRestriccionEmaxPasoDeTiempo);
  Result.addCampoDef('EmaxPasoDeTiempo', EmaxPasoDeTiempo);
  Result.addCampoDef('PagoPorDisponibilidad_USD_MWh',
    PagoPorDisponibilidad_USD_MWh, 98);
  Result.addCampoDef('PagoPorEnergia_USD_MWh', PagoPorEnergia_USD_MWh, 98);
  Result.addCampoDef('horasPurga', horasPurga, 189);
  Result.addCampoDef('horasTVPleno', horasTVPleno, 189);
  Result.addCampoDef('NDisc', NDisc, 189);
end;

procedure TFichaGTer_combinado_horario.BeforeRead(version, id_hilo: integer);
begin
  inherited BeforeRead(version, id_hilo);
  flg_TV_OnOffPorPaso := False;
  cv_NoCombustible_TG := 0.0;
  cv_NoCombustible_TV := 0.0;
end;

procedure TFichaGTer_combinado_horario.AfterRead(f: TArchiTexto);
begin
  inherited AfterRead(f);
  if f.version < 113 then
    flg_TV_OnOffPorPaso := False;

  if f.Version < 98 then
  begin
    PagoPorDisponibilidad_USD_MWh := 0;
    PagoPorEnergia_USD_MWh := 0;
  end;

  if f.Version < 98 then
  begin
    PagoPorDisponibilidad_USD_MWh := 0;
    PagoPorEnergia_USD_MWh := 0;
  end;

  if f.Version < 189 then
  begin
    horasPurga := 0;
    horasTVPleno := 0;
    NDisc := 0;
  end;
end;



procedure TFichaGTer_combinado_horario.generarLineaResumen(var archi: TextFile);
begin
  Write(archi, FloatToStrF(PMin_TG, formatoReales, 8, 1), #9,
    //PMín_TG
    FloatToStrF(PMax_TG, formatoReales, 8, 1), #9,
    //PMáx_TG
    FloatToStrF(cv_min_TG, formatoReales, 8, 2), #9,
    //CV_Mín_TG
    FloatToStrF(((cv_min_TG * PMin_TG + cv_TG * (PMax_TG - PMin_TG)) / PMax_TG),
    formatoReales, 8, 2),
    #9,    //CV_Medio_TG
    FloatToStrF(cv_TG, formatoReales, 8, 2), #9,
    //CV_Incremental_TG
    FloatToStrF(disp_TG, formatoReales, 8, 2), #9,
    //FDisp_TG
    '-', #9,
    //Costo Arranque
    '-', #9,
    //Costo Parada
    '-', #9,
    //mínNPasosOn
    '-', #9,
    //mínNPasosOff
    '-', #9,
    //desiciónPasosOnPorCiclo
    '-', #9,
    //desiciónPasosOffPorCiclo
    '-', #9,
    //costoPorCicloOn
    '-', #9);
  //costoPorCicloOff
end;

function TFichaGTer_combinado_horario.infoAd_: string;
begin
  Result := 'PMín_TG= ' + FloatToStrF(PMin_TG, ffGeneral, 10, 1) +
    ' MW, ' + 'PMáx_TG= ' + FloatToStrF(PMax_TG, ffGeneral, 10, 1) +
    ' MW, ' + 'cv_min_TG= ' + FloatToStrF(cv_min_TG, ffGeneral, 10, 1) +
    ' USD/MWh, ' + 'cv_TG= ' + FloatToStrF(cv_TG, ffGeneral, 10, 1) +
    ' USD/MWh, ' + 'fDisp_TG= ' + FloatToStrF(disp_TG, ffGeneral, 10, 2) +
    ' p.u., ' + 'tRep_TG= ' + FloatToStrF(tRepHoras_TG, ffGeneral, 10, 1) + 'h';
end;

procedure TFichaGTer_combinado_horario.Free;
begin
  inherited Free;
end;

//------------------------
// Métodos de TGTer_Combinado_horario
//========================
procedure TGTer_combinado_horario.dump_Variables(var f: TextFile; charIndentacion: char);
begin
  inherited dump_Variables(f, charIndentacion);
  writeln(f, charIndentacion, 'BMaxTG_SC[MW]= ', FloatToStrF(BMaxTG_SC, ffFixed, 10, 3));
  writeln(f, charIndentacion, 'BMaxTG_CC[MW]= ', FloatToStrF(BMaxTG_CC, ffFixed, 10, 3));
  writeln(f);
end;

class function TGTer_combinado_horario.TipoFichaLPD: TClaseDeFichaLPD;
begin
  Result := TFichaGTer_combinado_horario;
end;


procedure TGTer_combinado_horario.EvolucionarEstado;
begin
  inherited EvolucionarEstado;
  X_timer := Xs_timer;
end;

procedure TGTer_combinado_horario.PosicionarseEnEstrellita;
begin
  inherited PosicionarseEnEstrellita;
  X_timer := globs.CF.xr[ixr];
end;

procedure TGTer_combinado_horario.ActualizarEstadoGlobal(flg_Xs: boolean);
begin
  inherited ActualizarEstadoGlobal(flg_Xs);
  globs.CF.xr[ixr] := X_timer;
end;

procedure TGTer_combinado_horario.optx_nvxs(var ixr, ixd, iauxr, iauxd: integer);
begin
  inherited optx_nvxs(ixr, ixd, iauxr, iauxd);
  self.ixr := ixr;
  ixr := ixr + 1;
end;

procedure TGTer_combinado_horario.optx_RegistrarVariablesDeEstado(
  adminEstados: TAdminEstados);
begin
  inherited optx_RegistrarVariablesDeEstado(adminEstados);
  adminEstados.Registrar_Continua(
    ixr,
    0,
    5,
    pa.NDisc,
    nombre + '_timer', // nombre de la variable
    'h' // unidades
    );

end;

constructor TGTer_combinado_horario.Create(capa: integer; nombre: string;
  nacimiento, muerte: TFecha; lpdUnidades, lpd: TFichasLPD; nodo: TNodo;
  flg_CalcularGradienteDeInversion: boolean; TonCO2xMWh: NReal;
  LowCostMustRun, CleanDevelopmentMechanism: boolean; xFuenteIdxP: TFuenteAleatoria;
  xBorneIdxP: string; EstadoTimer: NReal);
begin
  inherited Create(capa, nombre, nacimiento, muerte, lpdUnidades, lpd, nodo,
    flg_CalcularGradienteDeInversion, TonCO2xMWh, LowCostMustRun,
    CleanDevelopmentMechanism,
    xFuenteIdxP, xBorneIdxP);
  self.EstadoTimer := EstadoTimer;
end;

function TGTer_combinado_horario.Rec: TCosa_RecLnk;
begin
  Result := inherited Rec;
  Result.addCampoDef('EstadoTimer', EstadoTimer, 190);
end;

procedure TGTer_combinado_horario.BeforeRead(version, id_hilo: integer);
begin
  inherited BeforeRead(version, id_hilo);
end;

procedure TGTer_combinado_horario.AfterRead(f: TArchiTexto);
begin
  inherited AfterRead(f);
end;

procedure TGTer_combinado_horario.PrepararMemoria(Catalogo: TCatalogoReferencias;
  globs: TGlobs);
begin
  inherited prepararMemoria(Catalogo, globs);
  NMaquinasDespachadasTG := 0;
  NMaquinasDespachadasTV := 0;
end;

procedure TGTer_combinado_horario.RegistrarParametrosDinamicos(
  CatalogoReferencias: TCatalogoReferencias);
var
  i: integer;
  ficha: TFichaGTer_combinado_horario;
begin
  inherited registrarParametrosDinamicos(CatalogoReferencias);
  lpd.expandirFichas(CatalogoReferencias, globs);
  lpd.RegistrarFichasAActualizar(Self, globs.ActualizadorLPD, @pA, nil);
  for i := 0 to lpd.Count - 1 do
  begin
    ficha := TFichaGTer_combinado_horario(lpd[i]);
    if ficha.indicePreciosPorCombustible <> nil then
      ficha.nroBornePreciosPorCombustible :=
        ficha.indicePreciosPorCombustible.IdBorne(ficha.bornePreciosPorCombustible);
  end;
end;

function TGTer_combinado_horario.PotenciaFirme: NReal;
var
  PTG, PTVa, PTVb: NReal;
begin
  PTG := paUnidades.nUnidades_Operativas[0] * pa.PMax_TG * pa.disp_TG;
  PTVa := paUnidades.nUnidades_Operativas[1] * pa.PMax_TV * pa.disp_TV;
  PTVb := PTG * pa.factorPotenciaTVdivTG;
  Result := PTG + min(PTVb, PTVa);
end;

function TGTer_combinado_horario.InfoAd_: string;
begin
  Result := '';
end;

class function TGTer_combinado_horario.DescClase: string;
begin
  Result := rsGeneradorTermicoCombinadoHorario;
end;

function TGTer_combinado_horario.get_pa_FD(kTipoUnidad: integer): NReal;
begin
  if kTipoUnidad = 0 then
    Result := pa.disp_TG
  else
    Result := pa.disp_TV;
end;

function TGTer_combinado_horario.get_pa_TMR(kTipoUnidad: integer): NReal;
begin
  if kTipoUnidad = 0 then
    Result := pa.tRepHoras_TG
  else
    Result := pa.tRepHoras_TV;
end;


procedure TGTer_combinado_horario.SorteosDelPaso(sortear: boolean);
begin
  if hayForzamientos or globs.ObligarDisponibilidad_1_ then
  begin
    NMaquinasDisponiblesTG := paUnidades.nUnidades_Operativas[0];
    NMaquinasDisponiblesTV := paUnidades.nUnidades_Operativas[1];
    PMaxDisponible_TG := pa.PMax_TG * NMaquinasDisponiblesTG;
    PMaxDisponible_TV := min(pa.PMax_TV * NMaquinasDisponiblesTV,
    PMaxDisponible_TG * pa.factorPotenciaTVdivTG);
  end
  else
  if sortear then
  begin
    if (globs.EstadoDeLaSala = CES_OPTIMIZANDO) then
    begin
      ActualizarProbabilidadesReparacionYRotura_(1, 0, 0);
      NMaquinasDisponiblesTG := paUnidades.nUnidades_Operativas[0];
      ActualizarProbabilidadesReparacionYRotura_(1, 0, 1);
      NMaquinasDisponiblesTV := paUnidades.nUnidades_Operativas[1];
    end
    else
    begin
      ActualizarProbabilidadesReparacionYRotura_(pa.disp_TG, pa.tRepHoras_TG, 0);
      NMaquinasDisponiblesTG := Sorteos_RepRotUnidades_k(0);
      ActualizarProbabilidadesReparacionYRotura_(pa.disp_TV, pa.tRepHoras_TV, 1);
      NMaquinasDisponiblesTV := Sorteos_RepRotUnidades_k(1);
    end;
    PMaxDisponible_TG := pa.PMax_TG * NMaquinasDisponiblesTG;
    PMaxDisponible_TV := min(pa.PMax_TV * NMaquinasDisponiblesTV,
    PMaxDisponible_TG * pa.factorPotenciaTVdivTG);
  end
  else
  begin
    NMaquinasDisponiblesTG := paUnidades.nUnidades_Operativas[0];
    NMaquinasDisponiblesTV := paUnidades.nUnidades_Operativas[1];
    PMaxDisponible_TG := pa.PMax_TG * NMaquinasDisponiblesTG * pa.disp_TG;
    PMaxDisponible_TV := min(pa.PMax_TV * NMaquinasDisponiblesTV,
      PMaxDisponible_TG * pa.factorPotenciaTVdivTG) * pa.disp_TV;
  end;

  PMaxDisponible_Central := PMaxDisponible_TG + PMaxDisponible_TV;
end;

procedure TGTer_combinado_horario.PrepararPaso_ps;
var
  indice: NReal;
begin
  {$IFDEF dbgc}
  if ( globs.EstadoDeLaSala =   CES_SIMULANDO) and (globs.kPaso_Sim = 19 ) then
     writeln('a');
  {$ENDIF}

  PMinTG := pa.PMin_TG;
  BMaxTG_SC := pa.PMax_TG - pa.PMin_TG;
  BMaxTG_CC := BMaxTG_SC;
  PMinTV := pa.PMin_TV;
  BMaxTV := pa.PMax_TV - pa.PMin_TV;
  PMaxTG := pa.PMax_TG;
  horasPurga := pa.horasPurga;
  horasTVPleno := pa.horasTVPleno;
  NDisc := pa.NDisc;
  //pendTV es la pendiente para prender a pleno la TV (que depende de las TG a traves de factorTV)
  pendTV := (PMaxTG - PMinTG) / (horasTVPleno);

  if pa.indicePreciosPorCombustible <> nil then
  begin
    indice := pa.indicePreciosPorCombustible.bornera[pa.nroBornePreciosPorCombustible];
    c0TG := pa.PMin_TG * (pa.cv_min_TG * indice + pa.cv_NoCombustible_TG);
    //este dsp va multiplicado por algo adimensionado (A)
    cvTG := pa.cv_TG * indice + pa.cv_NoCombustible_TG;
    //este va por algo con dimension MW (coef B)
  end
  else
  begin
    c0TG := pa.PMin_TG * (pa.cv_min_TG + pa.cv_NoCombustible_TG);
    cvTG := pa.cv_TG + pa.cv_NoCombustible_TG;
  end;
  c0TV := pa.PMin_TV * (pa.cv_min_TV + pa.cv_NoCombustible_TV);
  cvTV := pa.cv_TV + pa.cv_NoCombustible_TV;


  if (X_timer) < (horasPurga) then //tiempo purga
  begin
    factorTV := 0;
    purga_TV := 0;
  end
  else
  begin
    if (X_timer) < (horasTVPleno + horasPurga - 1) then //prendiendo TV despacito
    begin
      factorTV := pa.factorPotenciaTVdivTG;
      purga_TV := 1;
    end
    else
    begin
      factorTV := pa.factorPotenciaTVdivTG;
      purga_TV := 2;  //TV a pleno
    end;
  end;

  // Cálculo del al afectación del futuro por movimiento de estado.
  if X_timer < 5 then
  begin
    dCFdX := globs.CF.deltaCosto_vxr_continuo(ixr, globs.kPaso_Opt + 1, 1);
    {
  globs.CF.devxr_continuo(ixr, globs.kPaso_Opt + 1, dCFdX_Inc,
      dCFdX_Dec, resCode, xrpos);
  dCFdX := dCFdX_Inc;// (dCFdX_Inc + dCFdX_dec) / 2;
  }
  end
  else
  begin
    dCFdX := - globs.CF.deltaCosto_vxr_continuo(ixr, globs.kPaso_Opt + 1, -5);//dCFdX_Inc,
//    dCFdX_Dec, resCode, xrpos);

  end;

end;

procedure TGTer_combinado_horario.Sim_Paso_Fin;
var
  iposte: integer;
begin
  if NMaquinasDisponiblesTG > 0 then
  begin
    for iposte := 0 to high(P) do
      potMedia_despachada := potMedia_despachada + P[iposte] * globs.durpos[iposte];
    potMedia_despachada := potMedia_despachada * globs.invHorasDelPaso;
  end
  else
  begin
    potMedia_despachada := 0;
  end;
end;

procedure TGTer_combinado_horario.opt_nvers(var ivar, ivae, ires: integer);
begin
  if NMaquinasDisponiblesTG = 0 then
  begin
    exit;
  end;

  if NMaquinasDisponiblesTV = 0 then
  begin
    opt_nvers_TV_indisponible(ivar, ivae, ires);
  end
  else
  begin
    case purga_TV of
      0:
      begin
        opt_nvers_abierto_purga(ivar, ivae, ires);
      end;
      1, 2:
      begin
        opt_nvers_combinado(ivar, ivae, ires);
      end;
    end;
  end;


end;

procedure TGTer_combinado_horario.opt_nvers_TV_indisponible(var ivar, ivae,
  ires: integer);
begin

    //variables ATG_SC, BTG_SC, Xs_Timer
  Self.ivar := ivar;
  ivar := ivar + 3;

  ivar_ATG_SC := self.ivar;
  ivar_BTG_SC := ivar_ATG_SC + 1;

  ivar_Xs_timer := ivar_BTG_SC + 1;

  // Las variables ATG_SC, ATG_CC y Aacople son enteras
  Self.ivae := ivae;
  ivae := ivae + 1;

  Self.ires := ires;
  ires := ires + 3;
  // Están las restricciones de extinsión de BTG_SC < (PMax-PMin)*ATG_SC
  //BTG_CC < (PMax-PMin)*ATG_CC, BTV < (PMax-PMin)*ATV, Xs_Timer = AAcople*(X_timer + durPaso)
  // y también que ( B_TV + PMin_TV ) < f_TvxTG * (B_TG + PMin_TG )

  jres_dinTimer := self.ires;
  jres_Acople_BTG_SC := jres_dinTimer + 1;
  jres_Acople_TimerxBTG_SC := jres_Acople_BTG_SC + 1;

  jaux := jres_Acople_TimerxBTG_SC;

  if pA.HayRestriccionEmaxPasoDeTiempo then
  begin
    Inc(ires);
    jres_EMaxPasoDeTiempo := jaux + 1;
    jres_Ultima := jres_EMaxPasoDeTiempo;
  end
  else
  begin
    jres_Ultima := jaux + 1;
  end;

end;

procedure TGTer_combinado_horario.opt_nvers_abierto_purga(var ivar, ivae,
  ires: integer);
begin

  //variables ATG_SC, BTG_SC, ATG_CC, BTG_CC, AAcople, Xs_Timer
  Self.ivar := ivar;
  ivar := ivar + 5;

  ivar_ATG_SC := self.ivar;
  ivar_BTG_SC := ivar_ATG_SC + 1;

  ivar_ATG_CC := ivar_BTG_SC + 1;
 // ivar_BTG_CC := ivar_ATG_CC + 1;

  ivar_AAcople := ivar_ATG_CC + 1;
  ivar_Xs_timer := ivar_AAcople + 1;

  // Las variables ATG_SC, ATG_CC y Aacople son enteras
  Self.ivae := ivae;
  ivae := ivae + 3;

  Self.ires := ires;
  ires := ires + 4;
  // Están las restricciones de extinsión de BTG_SC < (PMax-PMin)*ATG_SC
  //BTG_CC < (PMax-PMin)*ATG_CC, BTV < (PMax-PMin)*ATV, Xs_Timer = AAcople*(X_timer + durPaso)
  // y también que ( B_TV + PMin_TV ) < f_TvxTG * (B_TG + PMin_TG )

  jres_dinTimer := self.ires;
  jres_Acople_BTG_SC := jres_dinTimer + 1;
  jres_Acople_TimerxBTG_SC := jres_Acople_BTG_SC + 1;
  jres_Acople_TimerxBTG_CC := jres_Acople_TimerxBTG_SC + 1;

  jaux := jres_Acople_TimerxBTG_CC;

  if pA.HayRestriccionEmaxPasoDeTiempo then
  begin
    Inc(ires);
    jres_EMaxPasoDeTiempo := jaux + 1;
    jres_Ultima := jres_EMaxPasoDeTiempo;
  end
  else
  begin
    jres_Ultima := jaux + 1;
  end;

end;

procedure TGTer_combinado_horario.opt_nvers_combinado(var ivar, ivae,
  ires: integer);
begin

    //variables ATG_SC, BTG_SC, ATG_CC, BTG_CC, ATV, BTV, AAcople, Xs_Timer
  Self.ivar := ivar;
  ivar := ivar + 8;

  ivar_ATG_SC := self.ivar;
  ivar_BTG_SC := ivar_ATG_SC + 1;

  ivar_ATG_CC := ivar_BTG_SC + 1;
  ivar_BTG_CC := ivar_ATG_CC + 1;

  ivar_ATV := ivar_BTG_CC + 1;
  ivar_BTV := ivar_ATV + 1;

  ivar_AAcople := ivar_BTV + 1;
  ivar_Xs_timer := ivar_AAcople + 1;

  // Las variables ATG_SC, ATG_CC, ATV y ATemp son enteras
  Self.ivae := ivae;
  ivae := ivae + 4;

  Self.ires := ires;
  ires := ires + 7;
  // Están las restricciones de extinsión de BTG_SC < (PMax-PMin)*ATG_SC
  //BTG_CC < (PMax-PMin)*ATG_SC, BTV < (PMax-PMin)*ATV, XTemp < Atemp*temp_max
  // y también que ( B_TV + PMin_TV ) < f_TvxTG * (B_TG + PMin_TG )

  jres_dinTimer := self.ires;
  jres_Acople_BTG_SC := jres_dinTimer + 1;
  jres_Acople_BTG_CC := jres_Acople_BTG_SC + 1;
  jres_Acople_BTV := jres_Acople_BTG_CC + 1;
  jres_Acople_TVxTG := jres_Acople_BTV + 1;
  jres_Acople_TimerxBTG_SC := jres_Acople_TVxTG + 1;
  jres_Acople_TimerxBTG_CC := jres_Acople_TimerxBTG_SC + 1;

  jaux := jres_Acople_TimerxBTG_CC;

  if pA.HayRestriccionEmaxPasoDeTiempo then
  begin
    Inc(ires);
    jres_EMaxPasoDeTiempo := jaux + 1;
    jres_Ultima := jres_EMaxPasoDeTiempo;
  end
  else
  begin
    jres_Ultima := jaux + 1;
  end;

end;

procedure TGTer_combinado_horario.opt_cargue(s: TSimplex);
begin

  if NMaquinasDisponiblesTG = 0 then
  begin
    exit;
  end;

  if NMaquinasDisponiblesTV = 0 then
  begin
    opt_cargue_TV_indisponible(s);
  end
  else
  begin
    case purga_TV of
      0:
      begin
        opt_cargue_abierto_purga(s);
      end;
      1, 2:
      begin
        opt_cargue_combinado(s);
      end;
    end;
  end;


end;

procedure TGTer_combinado_horario.opt_cargue_TV_indisponible(s: TSimplex);
var
   inodores: integer;
begin

  inodores := nodo.ires;
  // aporte a las restricciones del nodo
  s.pon_e(inodores, ivar_ATG_SC, PMinTG); // coef A TG sin calentar caldera
  s.pon_e(inodores, ivar_BTG_SC, 1); // coef de la B TG sin calentar caldera


  //restriccion de timer
  s.pon_e(jres_dinTimer, ivar_Xs_timer, -1);


  // coef B de las TG
  s.pon_e(jres_Acople_BTG_SC, ivar_ATG_SC, BMaxTG_SC);
  s.pon_e(jres_Acople_BTG_SC, ivar_BTG_SC, -1);

  //restriccion PTGSC menor que el max (cuando esta AAcople apagado)
  s.pon_e(jres_Acople_TimerxBTG_SC, ivar_ATG_SC, -1);
  s.pon_e(jres_Acople_TimerxBTG_SC, s.nc, NMaquinasDisponiblesTG + 0.1);

  //Restriccion a la energía máxima generable en el paso
  if pA.HayRestriccionEmaxPasoDeTiempo then
  begin
    //por ahora nada
  end;

  // aportes a la función de utilidad
  s.pon_e(s.nf, ivar_BTG_SC, -cvTG * globs.HorasDelPaso);
  s.pon_e(s.nf, ivar_ATG_SC, -c0TG * globs.HorasDelPaso);
  //  s.pon_e(s.nf, ivar_AAcople, -100 * globs.HorasDelPaso);

  {$IFNDEF INTERMEDIARIO_DE_COSTO_FUTURO}
  // Cargamos la afectación del costo futuro por movimiento del estado
  // -dC = -dCFdX ( Xs - X ) = -dCFdX * Xs + dCFdX * X
  s.pon_e(s.nf, ivar_Xs_timer, -dCFdX);
  s.acum_e(s.nf, s.nc, dCFdX * X_timer);
  {$ENDIF}

end;

procedure TGTer_combinado_horario.opt_cargue_abierto_purga(s: TSimplex);
var
   inodores: integer;
begin
    inodores := nodo.ires;
  // aporte a las restricciones del nodo
  s.pon_e(inodores, ivar_ATG_SC, PMinTG); // coef A TG sin calentar caldera
  s.pon_e(inodores, ivar_BTG_SC, 1); // coef de la B TG sin calentar caldera
  s.pon_e(inodores, ivar_ATG_CC, PMinTG);   // coef A TG calentando caldera
//  s.pon_e(inodores, ivar_BTG_CC, 1); // coef de la B TG calentando caldera

 // jres_tempXBTG := self.ires;
 // jres_Acople_BTG_SC := jres_tempXBTG + 1;
 // jres_Acople_BTG_CC := jres_Acople_BTG_SC + 1;
 // jres_Acople_BTV := jres_Acople_BTG_CC + 1;
 // jres_Acople_TVxTG := jres_Acople_BTV + 1;
 // jres_Acople_tempxBTG_SC := jres_Acople_TVxTG + 1;
 // jres_Acople_tempxBTG_CC := jres_Acople_tempxBTG_SC +1;

  //restriccion de timer
  s.pon_e(jres_dinTimer, ivar_AAcople, X_timer + globs.HorasDelPaso);
  s.pon_e(jres_dinTimer, ivar_Xs_timer, -1);
//  s.pon_e(jres_dinTimer, s.nc, 0.1);

  // coef B de las TG
  s.pon_e(jres_Acople_BTG_SC, ivar_ATG_SC, BMaxTG_SC);
  s.pon_e(jres_Acople_BTG_SC, ivar_BTG_SC, -1);

  //restriccion PTGSC menor que el max (cuando esta AAcople apagado)
  s.pon_e(jres_Acople_TimerxBTG_SC, ivar_ATG_SC, -1);
  s.pon_e(jres_Acople_TimerxBTG_SC, ivar_AAcople, -(NMaquinasDisponiblesTG));
  s.pon_e(jres_Acople_TimerxBTG_SC, s.nc, NMaquinasDisponiblesTG + 0.1);

  //restriccion PTGCC menor que el max (cuando esta AAcople prendido)
  s.pon_e(jres_Acople_TimerxBTG_CC, ivar_ATG_CC, -1);
  //  s.pon_e(jres_Acople_tempxBTG_CC, ivar_BTG_CC, -1);
  s.pon_e(jres_Acople_TimerxBTG_CC, ivar_AAcople, NMaquinasDisponiblesTG);

  //Restriccion a la energía máxima generable en el paso
  if pA.HayRestriccionEmaxPasoDeTiempo then
  begin
    //por ahora nada
  end;

  // aportes a la función de utilidad
  s.pon_e(s.nf, ivar_BTG_SC, -cvTG * globs.HorasDelPaso);
  s.pon_e(s.nf, ivar_ATG_SC, -c0TG * globs.HorasDelPaso);
 // s.pon_e(s.nf, ivar_BTG_CC, -cvTG * globs.HorasDelPaso);
  s.pon_e(s.nf, ivar_ATG_CC, -c0TG * globs.HorasDelPaso);
  //  s.pon_e(s.nf, ivar_AAcople, -100 * globs.HorasDelPaso);

  {$IFNDEF INTERMEDIARIO_DE_COSTO_FUTURO}
  // Cargamos la afectación del costo futuro por movimiento del estado
  // -dC = -dCFdX ( Xs - X ) = -dCFdX * Xs + dCFdX * X
  s.pon_e(s.nf, ivar_Xs_timer, -dCFdX);
  s.acum_e(s.nf, s.nc, dCFdX * X_timer);
  {$ENDIF}

end;

procedure TGTer_combinado_horario.opt_cargue_combinado(s: TSimplex);
var
   inodores: integer;
begin
  inodores := nodo.ires;
  // aporte a las restricciones del nodo
  s.pon_e(inodores, ivar_ATG_SC, PMinTG); // coef A TG sin calentar caldera
  s.pon_e(inodores, ivar_BTG_SC, 1); // coef de la B TG sin calentar caldera
  s.pon_e(inodores, ivar_ATG_CC, PMinTG);   // coef A TG calentando caldera
  s.pon_e(inodores, ivar_BTG_CC, 1); // coef de la B TG calentando caldera
  s.pon_e(inodores, ivar_ATV, PMinTV);   // coef A TG calentando caldera
  s.pon_e(inodores, ivar_BTV, 1); // coef de la B TG calentando caldera


 // jres_tempXBTG := self.ires;
 // jres_Acople_BTG_SC := jres_tempXBTG + 1;
 // jres_Acople_BTG_CC := jres_Acople_BTG_SC + 1;
 // jres_Acople_BTV := jres_Acople_BTG_CC + 1;
 // jres_Acople_TVxTG := jres_Acople_BTV + 1;
 // jres_Acople_tempxBTG_SC := jres_Acople_TVxTG + 1;
 // jres_Acople_tempxBTG_CC := jres_Acople_tempxBTG_SC +1;



  //restriccion de timer
  s.pon_e(jres_dinTimer, ivar_AAcople, X_timer + globs.HorasDelPaso);
  s.pon_e(jres_dinTimer, ivar_Xs_timer, -1);
//  s.pon_e(jres_dinTimer, s.nc, 0.1);

  // coef B de las TG
  s.pon_e(jres_Acople_BTG_SC, ivar_ATG_SC, BMaxTG_SC);
  s.pon_e(jres_Acople_BTG_SC, ivar_BTG_SC, -1);

  s.pon_e(jres_Acople_BTG_CC, ivar_ATG_CC, BMaxTG_CC);
  s.pon_e(jres_Acople_BTG_CC, ivar_BTG_CC, -1);

  // coef B[iposte] de las TV
  s.pon_e(jres_Acople_BTV, ivar_ATV, BMaxTV);
  s.pon_e(jres_Acople_BTV, ivar_BTV, -1);


  // restricción de acople de las centrales TG y TV
  if not hayForzamientos then
  begin
    s.pon_e(jres_Acople_TVxTG, ivar_ATG_CC, factorTV * PMinTG);
    s.pon_e(jres_Acople_TVxTG, ivar_BTG_CC, factorTV);
  end
  else
  begin
    s.pon_e(jres_Acople_TVxTG, ivar_ATG_CC, factorTV * PMinTG * 1.3);
    s.pon_e(jres_Acople_TVxTG, ivar_BTG_CC, factorTV * 1.3);
  end;
  s.pon_e(jres_Acople_TVxTG, ivar_ATV, -PMinTV);
  s.pon_e(jres_Acople_TVxTG, ivar_BTV, -1);

  //restriccion PTGSC menor que el max (cuando esta Atemp apagado)
  s.pon_e(jres_Acople_TimerxBTG_SC, ivar_ATG_SC, -1);
  //  s.pon_e(jres_Acople_tempxBTG_SC, ivar_BTG_SC, -1);
  s.pon_e(jres_Acople_TimerxBTG_SC, ivar_AAcople, -(NMaquinasDisponiblesTG));
  s.pon_e(jres_Acople_TimerxBTG_SC, s.nc, NMaquinasDisponiblesTG + 0.1);

  //restriccion PTGCC menor que el max (cuando esta Atemp prendido)
  s.pon_e(jres_Acople_TimerxBTG_CC, ivar_ATG_CC, -1);
  //  s.pon_e(jres_Acople_tempxBTG_CC, ivar_BTG_CC, -1);
  s.pon_e(jres_Acople_TimerxBTG_CC, ivar_AAcople, NMaquinasDisponiblesTG);



  //Restriccion a la energía máxima generable en el paso
  if pA.HayRestriccionEmaxPasoDeTiempo then
  begin
    //por ahora nada
  end;

  // aportes a la función de utilidad
  s.pon_e(s.nf, ivar_BTG_SC, -cvTG * globs.HorasDelPaso);
  s.pon_e(s.nf, ivar_ATG_SC, -c0TG * globs.HorasDelPaso);
  s.pon_e(s.nf, ivar_BTG_CC, -cvTG * globs.HorasDelPaso);
  s.pon_e(s.nf, ivar_ATG_CC, -c0TG * globs.HorasDelPaso);
  s.pon_e(s.nf, ivar_BTV, -cvTV * globs.HorasDelPaso);
  s.pon_e(s.nf, ivar_ATV, -c0TV * globs.HorasDelPaso);
  //  s.pon_e(s.nf, ivar_AAcople, -100 * globs.HorasDelPaso);

  {$IFNDEF INTERMEDIARIO_DE_COSTO_FUTURO}
  // Cargamos la afectación del costo futuro por movimiento del estado
  // -dC = -dCFdX ( Xs - X ) = -dCFdX * Xs + dCFdX * X
  s.pon_e(s.nf, ivar_Xs_timer, -dCFdX);
  s.acum_e(s.nf, s.nc, dCFdX * X_timer);
  {$ENDIF}
end;


procedure TGTer_combinado_horario.opt_fijarRestriccionesDeCaja(s: TSimplex);
var
  cotaBTG_CC: NReal;
begin

  if NMaquinasDisponiblesTG = 0 then
  begin
    exit;
  end;

  if NMaquinasDisponiblesTV = 0 then
  begin
    opt_fijarRestriccionesDeCaja_TV_indisponible(s);
  end
  else
  begin
    if (not hayForzamientos) then
    begin
      // Restricciones de caja de las B (Esto no es necesario pues las restricciones adicionales
      // obligan que estas se cumplan. Por eso le pongo el 1.1 para dejarle la caja un poco floja
      case purga_TV of
        0:
        begin
           opt_fijarRestriccionesDeCaja_abierto_purga(s);
        end;
        1:
        begin
           opt_fijarRestriccionesDeCaja_transicion(s);
        end;
        2:
        begin
           opt_fijarRestriccionesDeCaja_combinado(s);
        end;
      end;
    end;
  end;



end;

procedure TGTer_combinado_horario.opt_fijarRestriccionesDeCaja_TV_indisponible(
  s: TSimplex);
begin

  if NMaquinasDisponiblesTG = 0 then
  begin
    exit;
  end;

  s.cota_sup_set(ivar_Xs_timer, 5 * 2);
  s.FijarRestriccionIgualdad(jres_dinTimer);

  s.set_entera(ivae, ivar_ATG_SC, NMaquinasDisponiblesTG);

  if (not hayForzamientos) then
  begin
    s.cota_sup_set(ivar_BTG_SC, BMaxTG_SC * NMaquinasDisponiblesTG * 1.1);
  end;


end;

procedure TGTer_combinado_horario.opt_fijarRestriccionesDeCaja_abierto_purga(
  s: TSimplex);
begin

  s.cota_sup_set(ivar_Xs_timer, 5 * 2);
  s.FijarRestriccionIgualdad(jres_dinTimer);
  s.FijarRestriccionIgualdad(jres_Acople_TimerxBTG_CC);

  s.set_entera(ivae, ivar_ATG_SC, NMaquinasDisponiblesTG);
  s.set_entera(ivae + 1, ivar_ATG_CC, NMaquinasDisponiblesTG + 1);
  s.set_entera(ivae + 2, ivar_AAcople, 1);

//  if X_timer > AsumaCero then
//     s.FijarVariable(ivar_ATG_CC, NMaquinasDisponiblesTG);

  if (not hayForzamientos) then
  begin
    s.cota_sup_set(ivar_BTG_SC, BMaxTG_SC * NMaquinasDisponiblesTG * 1.1);
  end;


end;

procedure TGTer_combinado_horario.opt_fijarRestriccionesDeCaja_transicion(
  s: TSimplex);
var
   cotaBTG_CC: NReal;
begin
  s.cota_sup_set(ivar_Xs_timer, 5 * 3);
  s.FijarRestriccionIgualdad(jres_dinTimer);
  s.FijarRestriccionIgualdad(jres_Acople_TimerxBTG_CC);


  s.set_entera(ivae, ivar_ATG_SC, NMaquinasDisponiblesTG);
  s.set_entera(ivae + 1, ivar_ATG_CC, NMaquinasDisponiblesTG + 1);
  s.set_entera(ivae + 2, ivar_ATV, NMaquinasDisponiblesTV);
  s.set_entera(ivae + 3, ivar_AAcople, 1);

  if (not hayForzamientos) then
  begin
  s.cota_sup_set(ivar_BTG_SC, BMaxTG_SC * NMaquinasDisponiblesTG * 1.1);
  cotaBTG_CC := Min((pendTV * (X_timer - horasPurga) + PMinTG) *
    NMaquinasDisponiblesTG, BMaxTG_CC);
  s.cota_sup_set(ivar_BTG_CC, cotaBTG_CC);
  s.cota_sup_set(ivar_BTV, BMaxTV * NMaquinasDisponiblesTV * 1.1);
  end;


end;

procedure TGTer_combinado_horario.opt_fijarRestriccionesDeCaja_combinado(
  s: TSimplex);
begin

  s.cota_sup_set(ivar_Xs_timer, 5 * 3);
   s.FijarRestriccionIgualdad(jres_dinTimer);
   s.FijarRestriccionIgualdad(jres_Acople_TimerxBTG_CC);


   s.set_entera(ivae, ivar_ATG_SC, NMaquinasDisponiblesTG);
   s.set_entera(ivae + 1, ivar_ATG_CC, NMaquinasDisponiblesTG + 1);
   s.set_entera(ivae + 2, ivar_ATV, NMaquinasDisponiblesTV);
   s.set_entera(ivae + 3, ivar_AAcople, 1);

  if (not hayForzamientos) then
  begin
   s.cota_sup_set(ivar_BTG_SC, BMaxTG_SC * NMaquinasDisponiblesTG * 1.1);
   s.cota_sup_set(ivar_BTG_CC, BMaxTG_CC * NMaquinasDisponiblesTG * 1.1);
   s.cota_sup_set(ivar_BTV, BMaxTV * NMaquinasDisponiblesTV * 1.1);
  end;
end;


procedure TGTer_combinado_horario.opt_leerSolucion(s: TSimplex);
var
  ATG_SC, ATG_CC, ATV: integer;
  AAcople: integer;
  BTV, BTG_SC, BTG_CC: NReal;
  PDelPaso: NReal;
  EnergiaDespachada: NReal;
begin

  Ingreso_PorDisponibilidad_ :=
    PMaxDisponible_Central * pa.PagoPorDisponibilidad_USD_MWh * globs.HorasDelPaso;

  CostoDirectoDelPaso :=0;
  Ingreso_PorEnergia_:= 0;

  if NMaquinasDisponiblesTG = 0 then
  begin
    vclear(P);
    NMaquinasDespachadasTG := 0;
    NMaquinasDespachadasTV := 0;
    Xs_timer := 0;
    exit;
  end
  else if NMaquinasDisponiblesTV = 0 then
  begin
     opt_leerSolucion_TV_indisponible(s);
  end
  else
  begin
    case purga_TV of
      0:
      begin
         opt_leerSolucion_abierto_purga(s);
      end;
      1,2:
      begin
         opt_leerSolucion_combinado(s);
      end;
    end;
  end;


end;

procedure TGTer_combinado_horario.opt_leerSolucion_TV_indisponible(s: TSimplex);
var
  ATG_SC: integer;
  BTV, BTG_SC: NReal;
  PDelPaso: NReal;
  EnergiaDespachada: NReal;

begin
 Xs_timer :=  s.xval( ivar_Xs_timer );
 if Xs_timer >= (5*0.9) then
      Xs_timer := 5
 else
 if Xs_timer <= AsumaCero then
      Xs_timer := 0;

 EnergiaDespachada := 0;

   ATG_SC := trunc(s.xval(ivar_ATG_SC) + 0.2);

   NMaquinasDespachadasTG := ATG_SC;
   NMaquinasDespachadasTV := 0;

   PDelPaso := 0;
   if ATG_SC > 0 then
   begin
     BTG_SC := s.xval(ivar_BTG_SC);
     PDelPaso := BTG_SC + ATG_SC * PMinTG;
   end
   else
     BTG_SC := 0;

   EnergiaDespachada := EnergiaDespachada + PDelPaso * globs.HorasDelPaso;
   P[0] := PDelPaso;   //Como este modelo siempre es horario esta bien esto

   if NMaquinasDespachadasTG > 0 then
   begin
     CostoDirectoDelPaso := ( BTG_SC * cvTG + ATG_SC * c0TG) *
       globs.HorasDelPaso;  //100 * AAcople +
   end
   else
     CostoDirectoDelPaso := 0;

 Ingreso_PorEnergia_ := EnergiaDespachada * pa.PagoPorEnergia_USD_MWh;

end;

procedure TGTer_combinado_horario.opt_leerSolucion_abierto_purga(s: TSimplex);
var
  ATG_SC, ATG_CC, ATV: integer;
  AAcople: integer;
  BTV, BTG_SC, BTG_CC: NReal;
  PDelPaso: NReal;
  EnergiaDespachada: NReal;

begin
 Xs_timer :=  s.xval( ivar_Xs_timer );
 if Xs_timer >= (5*0.9) then
      Xs_timer := 5
 else
 if Xs_timer <= AsumaCero then
      Xs_timer := 0;

 EnergiaDespachada := 0;

   ATG_SC := trunc(s.xval(ivar_ATG_SC) + 0.2);
   ATG_CC := trunc(s.xval(ivar_ATG_CC) + 0.2);
   AAcople := trunc(s.xval(ivar_AAcople) + 0.2);

   NMaquinasDespachadasTG := ATG_SC + ATG_CC;
   NMaquinasDespachadasTV := 0;

   PDelPaso := 0;
   if ATG_SC > 0 then
   begin
     BTG_SC := s.xval(ivar_BTG_SC);
     PDelPaso := BTG_SC + ATG_SC * PMinTG;
   end
   else
     BTG_SC := 0;

   if ATG_CC > 0 then
   begin
     PDelPaso := ATG_CC * PMinTG;
   end;

   EnergiaDespachada := EnergiaDespachada + PDelPaso * globs.HorasDelPaso;
   P[0] := PDelPaso;   //Como este modelo siempre es horario esta bien esto

   if NMaquinasDespachadasTG > 0 then
   begin
     CostoDirectoDelPaso := ( BTG_SC * cvTG + ATG_SC * c0TG + ATG_CC * c0TG) *
       globs.HorasDelPaso;  //100 * AAcople +
   end
   else
     CostoDirectoDelPaso := 0;

 Ingreso_PorEnergia_ := EnergiaDespachada * pa.PagoPorEnergia_USD_MWh;
end;

procedure TGTer_combinado_horario.opt_leerSolucion_combinado(s: TSimplex);
var
  ATG_SC, ATG_CC, ATV: integer;
  AAcople: integer;
  BTV, BTG_SC, BTG_CC: NReal;
  PDelPaso: NReal;
  EnergiaDespachada: NReal;
begin
  Xs_timer :=  s.xval( ivar_Xs_timer );
  if Xs_timer >= (5*0.9) then
       Xs_timer := 5
  else
  if Xs_timer <= AsumaCero then
       Xs_timer := 0;

  EnergiaDespachada := 0;

    ATG_SC := trunc(s.xval(ivar_ATG_SC) + 0.2);
    ATG_CC := trunc(s.xval(ivar_ATG_CC) + 0.2);
    ATV := trunc(s.xval(ivar_ATV) + 0.2);
    AAcople := trunc(s.xval(ivar_AAcople) + 0.2);

    NMaquinasDespachadasTG := ATG_SC + ATG_CC;
    NMaquinasDespachadasTV := ATV;

    PDelPaso := 0;
    if ATG_SC > 0 then
    begin
      BTG_SC := s.xval(ivar_BTG_SC);
      PDelPaso := BTG_SC + ATG_SC * PMinTG;
    end
    else
      BTG_SC := 0;

    if ATG_CC > 0 then
    begin
      BTG_CC := s.xval(ivar_BTG_CC);
      PDelPaso := BTG_CC + ATG_CC * PMinTG;
    end
    else
      BTG_CC := 0;

    if ATV > 0 then
    begin
      BTV := s.xval(ivar_BTV);
      PDelPaso := PDelPaso + BTV + ATV * PMinTV;
    end
    else
      BTV := 0;

    EnergiaDespachada := EnergiaDespachada + PDelPaso * globs.HorasDelPaso;
    P[0] := PDelPaso;   //Como este modelo siempre es horario esta bien esto

    if NMaquinasDespachadasTG > 0 then
    begin
      CostoDirectoDelPaso := ( BTG_SC * cvTG + BTG_CC * cvTG + ATG_SC * c0TG + ATG_CC * c0TG + BTV * cvTV + ATV * c0TV) *
        globs.HorasDelPaso;  //100 * AAcople +
    end
    else
      CostoDirectoDelPaso := 0;

  Ingreso_PorEnergia_ := EnergiaDespachada * pa.PagoPorEnergia_USD_MWh;

end;

procedure TGTer_combinado_horario.Sim_Cronica_Inicio;
begin
  inherited Sim_Cronica_Inicio;
  X_timer := EstadoTimer;
  Xs_timer := X_timer;
end;

function TGTer_combinado_horario.getNombreVar(ivar: integer; var nombre: string): boolean;
begin

  if (NMaquinasDisponiblesTG = 0) then
  begin
    Result := False;
  end
  else if (NMaquinasDisponiblesTV = 0) then
  begin
    getNombreVar_TV_indisponible(ivar, nombre);
  end
  else
  begin
    case purga_TV of
       0:
       begin
          getNombreVar_abierto_purga(ivar, nombre);
       end;
       1, 2:
       begin
          getNombreVar_combinado(ivar, nombre);
       end;
     end;
  end;



end;

function TGTer_combinado_horario.getNombreVar_TV_indisponible(ivar: integer;
  var nombre: string): boolean;
begin


  if (ivar < ivar_ATG_SC) or (ivar >= (ivar_Xs_timer + 1)) then
  begin
      Result := False;
  end
  else
  begin
    if ivar < ivar_BTG_SC then
      nombre := self.Nombre + '_A_TG_SC[MW]' + IntToStr(ivar - ivar_ATG_SC + 1)
    else if ivar < ivar_Xs_timer then
      nombre := self.Nombre + '_B_TG_SC' + IntToStr(ivar - ivar_BTG_SC + 1)
    else
      nombre := self.Nombre + '_Xs_timer' + IntToStr(ivar - ivar_Xs_timer + 1);

    Result := True;
  end;

end;

function TGTer_combinado_horario.getNombreVar_abierto_purga(ivar: integer;
  var nombre: string): boolean;
begin

  if (NMaquinasDisponiblesTG = 0) then
  begin
    Result := False;
  end
  else if (ivar < ivar_ATG_SC) or (ivar >= (ivar_Xs_timer + 1)) then
  begin
      Result := False;
  end
  else
  begin
    if ivar < ivar_BTG_SC then
      nombre := self.Nombre + '_A_TG_SC[MW]' + IntToStr(ivar - ivar_ATG_SC + 1)
    else if ivar < ivar_ATG_CC then
      nombre := self.Nombre + '_B_TG_SC' + IntToStr(ivar - ivar_BTG_SC + 1)
    else if ivar < ivar_AAcople then
      nombre := self.Nombre + '_A_TG_CC[MW]' + IntToStr(ivar - ivar_ATG_CC + 1)
    else if ivar < ivar_Xs_timer then
      nombre := self.Nombre + '_AAcople' + IntToStr(ivar - ivar_AAcople + 1)
    else
      nombre := self.Nombre + '_Xs_timer' + IntToStr(ivar - ivar_Xs_timer + 1);

    Result := True;
  end;

end;

function TGTer_combinado_horario.getNombreVar_combinado(ivar: integer;
  var nombre: string): boolean;
begin

  if (NMaquinasDisponiblesTG = 0) then
  begin
    Result := False;
  end
  else if (ivar < ivar_ATG_SC) or (ivar >= (ivar_Xs_timer + 1)) then
  begin
      Result := False;
  end
  else
  begin
    if ivar < ivar_BTG_SC then
      nombre := self.Nombre + '_A_TG_SC[MW]' + IntToStr(ivar - ivar_ATG_SC + 1)
    else if ivar < ivar_ATG_CC then
      nombre := self.Nombre + '_B_TG_SC' + IntToStr(ivar - ivar_BTG_SC + 1)
    else if ivar < ivar_BTG_CC then
      nombre := self.Nombre + '_A_TG_CC[MW]' + IntToStr(ivar - ivar_ATG_CC + 1)
    else if ivar < ivar_ATV then
      nombre := self.Nombre + '_B_TG_CC' + IntToStr(ivar - ivar_BTG_CC + 1)
    else if ivar < ivar_BTV then
      nombre := self.Nombre + '_ATV[MW]' + IntToStr(ivar - ivar_ATV + 1)
    else if ivar < ivar_AAcople then
      nombre := self.Nombre + '_BTV' + IntToStr(ivar - ivar_BTV + 1)
    else if ivar < ivar_Xs_timer then
      nombre := self.Nombre + '_AAcople' + IntToStr(ivar - ivar_AAcople + 1)
    else
      nombre := self.Nombre + '_Xs_timer' + IntToStr(ivar - ivar_Xs_timer + 1);

    Result := True;
  end;

end;

function TGTer_combinado_horario.getNombreRes(ires: integer; var nombre: string): boolean;
begin

  if (NMaquinasDisponiblesTG = 0) then
  begin
    Result := False;
  end
  else if (NMaquinasDisponiblesTV = 0) then
  begin
    getNombreVar_TV_indisponible(ivar, nombre);
  end
  else
  begin
    case purga_TV of
       0:
       begin
          getNombreRes_abierto_purga(ires, nombre);
       end;
       1, 2:
       begin
          getNombreRes_combinado(ires, nombre);
       end;
     end;
  end;

end;

function TGTer_combinado_horario.getNombreRes_TV_indisponible(ires: integer;
  var nombre: string): boolean;
begin

  if (ires < jres_dinTimer) or (ires > jres_Ultima) then
  begin
      Result := False;
  end
  else
  begin
    if ires < jres_Acople_BTG_SC then
      nombre := self.nombre + '_res-dinTimer' + IntToStr(ires - jres_dinTimer + 1)
    else if ires < jres_Acople_TimerxBTG_SC then
      nombre := self.nombre + '_res-extincion_BTG_SC' + IntToStr(ires - jres_Acople_BTG_SC + 1)
    else if ires < jres_Ultima then
      nombre := self.nombre + '_res-Acople_TimerxBTG_SC' + IntToStr(ires - jres_Acople_TimerxBTG_SC + 1)
    else if (pa.HayRestriccionEmaxPasoDeTiempo) then
    begin
    end;//por ahora nada
    Result := True;
  end;

end;

function TGTer_combinado_horario.getNombreRes_abierto_purga(ires: integer;
  var nombre: string): boolean;
begin
 {
 jres_dinTimer := self.ires;
 jres_Acople_BTG_SC := jres_dinTimer + 1;
 jres_Acople_TimerxBTG_SC := jres_Acople_BTG_SC + 1;
 jres_Acople_TimerxBTG_CC := jres_Acople_TimerxBTG_SC + 1;
 }

    if (NMaquinasDisponiblesTG = 0) then
    begin
         Result := False;
    end
    else  if (ires < jres_dinTimer) or (ires > jres_Ultima) then
    begin
        Result := False;
    end
    else
    begin
      if ires < jres_Acople_BTG_SC then
        nombre := self.nombre + '_res-dinTimer' + IntToStr(ires - jres_dinTimer + 1)
      else if ires < jres_Acople_TimerxBTG_SC then
        nombre := self.nombre + '_res-extincion_BTG_SC' + IntToStr(ires - jres_Acople_BTG_SC + 1)
      else if ires < jres_Acople_TimerxBTG_CC then
        nombre := self.nombre + '_res-Acople_TimerxBTG_SC' + IntToStr(ires - jres_Acople_TimerxBTG_SC + 1)
      else if ires < jres_Ultima then
      nombre := self.nombre + '_res-Acople_TimerxBTG_CC' + IntToStr(ires - jres_Acople_TimerxBTG_CC + 1)
      else if (pa.HayRestriccionEmaxPasoDeTiempo) then
      begin
      end;//por ahora nada
      Result := True;
    end;

end;

function TGTer_combinado_horario.getNombreRes_combinado(ires: integer;
  var nombre: string): boolean;
begin

    if (NMaquinasDisponiblesTG = 0) then
    begin
         Result := False;
    end
    else  if (ires < jres_dinTimer) or (ires > jres_Ultima) then
    begin
        Result := False;
    end
    else
    begin
      if ires < jres_Acople_BTG_SC then
        nombre := self.nombre + '_res-dinTimer' + IntToStr(ires - jres_dinTimer + 1)
      else if ires < jres_Acople_BTG_CC then
        nombre := self.nombre + '_res-Acople_BTG_SC' + IntToStr(ires - jres_Acople_BTG_SC + 1)
      else if ires < jres_Acople_BTV then
        nombre := self.nombre + '_res-Acople_BTG_CC' + IntToStr(ires - jres_Acople_BTG_CC + 1)
      else if ires < jres_Acople_TVxTG then
        nombre := self.nombre + '_res-Acople_BTV' + IntToStr(ires - jres_Acople_BTV + 1)
      else if ires < jres_Acople_TimerxBTG_SC then
        nombre := self.nombre + '_res-Acople_TVxTG' + IntToStr(ires - jres_Acople_TVxTG + 1)
      else if ires < jres_Acople_TimerxBTG_CC then
        nombre := self.nombre + '_res-Acople_TimerxBTG_SC' + IntToStr(ires - jres_Acople_TimerxBTG_SC + 1)
      else if ires < jres_Ultima then
      nombre := self.nombre + '_res-Acople_TimerxBTG_CC' + IntToStr(ires - jres_Acople_TimerxBTG_CC + 1)
      else if (pa.HayRestriccionEmaxPasoDeTiempo) then
      begin
      end;//por ahora nada
      Result := True;
    end;

end;

procedure TGTer_combinado_horario.PubliVars;
begin
  inherited PubliVars;
  PublicarVariableNI('NDespachadasTGs', '-', NMaquinasDespachadasTG, True);
  PublicarVariableNI('NDespachadasTVs', '-', NMaquinasDespachadasTV, True);

  {$IFDEF DECLARAR_VARIABLES_SIMRES_DEF}
  declararVarsPSimResPorDefectoIntercalandoPostes(['P', 'Costo', 'NMaqsDespachadas'],
    globs.NPostes);
  {$ENDIF}
  PublicarVariableNR('c0TG', '[USD/h]', 6, 1, c0TG, False);
  PublicarVariableNI('NMaqsDisponiblesTG', '-', NMaquinasDisponiblesTG, False);
  PublicarVariableNI('NMaqsDisponiblesTV', '-', NMaquinasDisponiblesTV, False);
  PublicarVariableNR('PMaxDisponible_Central', '[MW]', 6, 1,
    PMaxDisponible_Central, False);
  PublicarVariableNR('PMediaDespachada', '[MW]', 6, 1, potMedia_despachada, False);
  PublicarVariableNR('Timer', '[h]', 6, 1, self.X_timer, True);
end;

procedure TGTer_combinado_horario.Free;
begin
  inherited Free;
end;


procedure AlInicio;
begin
  registrarClaseDeCosa(TGTer_combinado_horario.ClassName, TGTer_combinado_horario);
  registrarClaseDeCosa(TFichaGTer_combinado_horario.ClassName, TFichaGTer_combinado_horario);
end;

procedure AlFinal;
begin
end;

end.
